;+
; NAME:
;     mw_explore
; PURPOSE:
;     to interactively explore the brightness temperature maps and
;     spectra of a model produced by MW.
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mw_explore[,infile][,out=out,fghz=fghz,pos=pos]
; INPUTS:
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;     infile    output of mw.pro containing the brightness
;                 temperature maps.  If omitted, the user
;                 can choose one interactively
; ROUTINES CALLED:
; OUTPUTS:
;     out	optional output, the R and L spectrum (as a complex
;             array with float=R and imaginary=L).  This output
;             is activated by clicking with RIGHT mouse button
;             to exit.
;     fghz  optional output, the array of frequencies, in GHz.
;             This output is activated by clicking with RIGHT
;             mouse button to exit.
;     pos   optional output, the x,y index values at which the
;             OUT spectrum was obtained.
;     Two plots are produced, one showing a map, over which the
;     mouse can be moved to specify the point to show the
;     spectrum, and the other showing the X- and O-mode
;     spectra as solid and dashed lines, respectively.
; COMMENTS:
;   1. To start, click in the map and then move the mouse.
;	2. If the user puts the mouse over the bar at the bottom,
;        a different frequency can be chosen as the gray-scale
;        map.
;	3. To exit, move the mouse off the left edge of the plot or
;        click with the RIGHT mouse button.
;	4. The longitudinal B field is overploted as contours.
; SIDE EFFECTS:
; RESTRICTIONS:
; MODIFICATION HISTORY:
;     Written 09-Mar-2003 by Dale E. Gary
;     13-Apr-2003  DG
;       Tried to make it more general, and made code somewhat
;       simpler.
;-
pro mw_explore,infile,out=out,fghz=fghz,pos=pos

   ; If INFILE is not defined, ask the user for a file.
   if(n_elements(infile) eq 0) then infile = dialog_pickfile(filter='*.sav')
   if (infile eq '') then return

   ; Restore the file and get some info on sizes
   restore,infile
   nx = n_elements(x)
   ny = n_elements(y)
   nf = n_elements(fghz)

   ; Open a pixmap window for the hidden image window
   window,/free,/pixmap,xsiz=500,ysiz=400
   pw1 = !d.window

   ; Define and open with image window
   window,/free,xsiz=500,ysiz=400
   w1 = !d.window

   ; Plot the x-mode brightness temperature map for the middle frequency
   imap = nf/2
   fdisp = string(fghz[imap],format='(F5.2," GHz")')
   isrc = 0     ; Specifies whether to plot r(=0), l(=1), s(=2) or p(=3)
   r = float(tb2)      ; RCP
   l = imaginary(tb2)  ; LCP
   s = (r+l)/2.        ; Total intensity
   p = (r-l)/(r+l)     ; Polarization
   contour,r[*,*,imap],x,y,/fill,nlev=100,xsty=1,ysty=1
   ; Overplot the longitudinal B field
   v = [-1000,-500,-200,-100,0,100,200,500,1000]
   if (n_elements(bz0) ne 0) then contour,/over,bz0,x,y,lev=v,c_linesty=v lt 0
   ; Plot a little grid along the bottom to help pick a frequency
   gridx_min = -80
   gridy_min = -60
   gridx = [0,0,1,1,2,2,3,3,4,4,5,5,4,4,3,3,2,2,1,1,0]*20+gridx_min
   gridy = [0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0]*5+gridy_min
   plots,gridx,gridy
   xyouts,gridx_min,gridy_min,string(nint(fghz[0]),format='(I2)')    ; Minimum freq
   xyouts,gridx_min+95,gridy_min,string(nint(fghz[nf-1]),format='(I2)')  ; Maximum freq

   ; Copy the visible image to the hidden window
   wset,pw1
   device,copy=[0,0,500,400,0,0,w1]

   ; Open a pixmap window for the hidden spectral plot
   window,/free,/pixmap,xsiz=300,ysiz=500
   pw2 = !d.window

   ; Open the visible window for the spectral plot
   window,/free,xsiz=300,ysiz=500
   w2 = !d.window

   ; Get a mouse click to start things off
   wset,w1
   cursor,cx,cy,/wait
   ; As long as the mouse is on the plot, loop over the mouse position
   while (1) do begin
      cursor,cx,cy,/nowait
      if (!mouse.button eq 2) then begin
         isrc = (isrc+1) mod 4
         wset,pw1
         ; Display the map in the hidden window and remake the grid on the bottom
         case isrc of
            0: contour,r[*,*,imap],x,y,/fill,nlev=100,/xsty,/ysty,title='RCP '+fdisp
            1: contour,l[*,*,imap],x,y,/fill,nlev=100,/xsty,/ysty,title='LCP '+fdisp
            2: contour,s[*,*,imap],x,y,/fill,nlev=100,/xsty,/ysty,title='Intensity '+fdisp
            3: contour,p[*,*,imap],x,y,/fill,nlev=100,/xsty,/ysty,title='Polarization '+fdisp
            else: contour,r[*,*,imap],x,y,/fill,nlev=100,/xsty,/ysty,title='RCP '+fdisp
         endcase
         if (n_elements(bz0) ne 0) then contour,/over,bz0,x,y,lev=v,c_linesty=v lt 0
         plots,gridx,gridy
         xyouts,gridx_min,gridy_min,string(nint(fghz[0]),format='(I2)')    ; Minimum freq
         xyouts,gridx_min+95,gridy_min,string(nint(fghz[nf-1]),format='(I2)')  ; Maximum freq

         ; Copy the image to the visible window
         wset,w1
         device,copy=[0,0,500,400,0,0,pw1]
      endif

      ; Plot the spectra in the pixmap window
      wset,pw2
      junk = min(abs(x-cx),ix)  ; convert world mouse x coord to map index
      junk = min(abs(y-cy),iy)  ; convert world mouse y coord to map index
      ix = ix>0<(nx-1) & iy = iy>0<(ny-1)  ; Ensure coords are on the map
      ; Plot the x-mode spectrum
      plot,fghz,r[ix,iy,*],$
              /xlog,xran=[1.,25.],xsty=1,$
              /ylog,yran=[1.E4,3.E6],Ysty=1,$
              title=string(x[ix],y[iy],format='("Location: ",F4.0,",",F4.0)')
      ; plot the o-mode spectrum
      oplot,fghz,imaginary(tb2[ix,iy,*]),linesty=2
      ; plot a couple of markers to show which map is displayed
      plots,fghz[imap],r[ix,iy,imap],psym=2
      plots,fghz[imap],l[ix,iy,imap],psym=4

      ; Move the spectral plot to the visible window
      wset,w2
      device,copy=[0,0,300,500,0,0,pw2]

      ; Exit with cursor location and extraction of spectra if right
      ; mouse button was clicked
      if (!mouse.button eq 4) then goto,out

      ; Have to reset the map window world coordinates
      wset,w1
      if (cy le -55) then begin
         ; The mouse is on the grid at the bottom, so select a new map to display
         imap = fix((cx-gridx_min)*nf/100.)>0<(nf-1)
         fdisp = string(fghz[imap],format='(F5.2," GHz")')

         wset,pw1
         ; Display the map in the hidden window and remake the grid on the bottom
         case isrc of
            0: contour,r[*,*,imap],x,y,/fill,nlev=100,/xsty,/ysty,title='RCP '+fdisp
            1: contour,l[*,*,imap],x,y,/fill,nlev=100,/xsty,/ysty,title='LCP '+fdisp
            2: contour,s[*,*,imap],x,y,/fill,nlev=100,/xsty,/ysty,title='Intensity '+fdisp
            3: contour,p[*,*,imap],x,y,/fill,nlev=100,/xsty,/ysty,title='Polarization '+fdisp
            else: contour,r[*,*,imap],x,y,/fill,nlev=100,/xsty,/ysty,title='RCP '+fdisp
         endcase
         if (n_elements(bz0) ne 0) then contour,/over,bz0,x,y,lev=v,c_linesty=v lt 0
         plots,gridx,gridy
         xyouts,gridx_min,gridy_min,string(nint(fghz[0]),format='(I2)')    ; Minimum freq
         xyouts,gridx_min+95,gridy_min,string(nint(fghz[nf-1]),format='(I2)')  ; Maximum freq

         ; Copy the image to the visible window
         wset,w1
         device,copy=[0,0,500,400,0,0,pw1]
      endif else begin
         ; The mouse is just in the main window, so reset the map window world coords
         ; to prepare for another cursor call
         contour,r[*,*,imap],x,y,/nodata,/noerase,/xsty,/ysty
      endelse
   endwhile
out:
   if (!mouse.button eq 4) then begin
      posx=ix>0<(nx-1)
      posy=iy>0<(ny-1)
      out = reform(tb2[posx,posy,*])
      pos = [posx,posy]
   endif

   ; All done, so delete the pixmap windows
   wdelete,pw1
   wdelete,pw2
return
end