;+
; NAME:
;     mw_ff_opacity
; PURPOSE:
;     calculates (incremental) opacities due to free-free absorption
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mw_ff_opacity,model,fGHz
; INPUTS:
;     model   	structure containing model values
;     fGHz      frequency array in [GHz]
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
; ROUTINES CALLED:
; OUTPUTS:
;     ff	structure containing quantities of f-f opacity
; COMMENTS:
;     Free-free opacity expression is taken from
;     Schmelz et al. 1994..apj..434..786 (Actually, Lang 1980)
;      tau_ff_X,O = 9.8E-3 (f -/+ 2.8E6 Bcos a)^-2  Te^-1.5
;		nv^-1 ln(4.7E10 Te/f) \int ne^2 dL
;     where a=viewing angle, nv=the index of refrac
;     all in c.g.s units, -ve = the X mode.
;     cf. Dulk's expression:
; 	kappa=alog(0.21)-2.*alog(fHz)-1.5*alog(te3d)
;          +2*alog(ne3d)+alog(xscale)
; SIDE EFFECTS:
; RESTRICTIONS:
;     1. Keep |Bz|<3500G for f>1GHz NOT to make (fGHz-f_Bz) negative
;     2. calculate opacity only upto (nz-2)-th layer, due to definition
;        the thickness of each layer or def. of incremental opacity
;	 (see 00readme for more details). This is quivalent to assuming
;        tau=0 at the top layer.
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)
;     09-Mar-2003  DG
;       Changed to put fastest varying indexes first to avoid maniacal
;       page faulting.
;-

function mw_ff_opacity,mdl,fGHz

Te =mdl.Te 		; [K]
Dn =mdl.Dn 		; [cm-3]
Bz =mdl.Bz>(-350.)<350. 	; [gauss] - see RESTRICTIONS
z  =mdl.z * 1.0E5 	; [cm]

nx =n_elements(mdl.x)
ny =n_elements(mdl.y)
nz =n_elements(z)
nf =n_elements(fGHz)

Te3f =fltarr(nf,nz,nx,ny)
tau_x=fltarr(nf,nz,nx,ny)
tau_o=fltarr(nf,nz,nx,ny)

for k=0,nz-2 do begin ; Why not nz-1? See RESTRICTIONS

   T2d=reform( te[*,*,k])>1.0	; temperature
   B2d=reform( Bz[*,*,k])	; Bz
   D2d=reform( Dn[*,*,k])>1.0 	; density
   hh=max(z[k+1]-z[k]) ; geometrical thickness

; frequency-independent part
   ham=alog(9.8E-3)-1.5*alog(T2d)+2.*alog(D2d)+alog(hh)

; now simply add the frequency-dependent part
   for q=0,nf-1 do begin
    fHz=fGHz[q]*1.0E9
    hama=ham+alog(24.5734+alog(T2d/fHz))
    tau_x[q,k,*,*] = hama - 2.0*alog( fHz - 2.8E6*abs(B2d) )
    tau_o[q,k,*,*] = hama - 2.0*alog( fHz + 2.8E6*abs(B2d) )
     Te3f[q,k,*,*] = T2d
   endfor ; next frequency

endfor ; - next k

tau_x=exp(tau_x>(-66.)<66.)
tau_o=exp(tau_o>(-66.)<66.)

ff ={Te:Te3f, tau_x:tau_x, tau_o:tau_o}

return, ff

end
