;+
; NAME:
;     mw_gr_opacity
; PURPOSE:
;     calculates the opacities due to gyroresonant absorption
;	upto harmponic=6
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mw_gr_opacity,model,fGHz,fk
; INPUTS:
;     mdl   structure containing model values
;     fGHz  frequency array in [GHz]
;     fk	floating arr for indices matching the height of
;		      gyroresonant layers
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
; ROUTINES CALLED:
; OUTPUTS:
;     gr	structure containing all quantities related to
;             the gyroresonant opacity
; COMMENTS:
;     Gyroresonant opacity expression taken from
;	Schmelz et al. 1994..apj..434..786
;     tau_gr_X,O = (0.052/2.) (s^2s/2^s.s!) ne f^-1 LB
;	x (1.77E-10 Te)^(s-1) (1 +- cos a)^2 (sin a)^(2s-2)
;     where a=viewing angle, +ve=x-mode, all c.g.s units
;           Te[K], Dn[cm-3], B[Gauss], z[cm].
;     Brosius used half of this value, not included here.
;     Use precalculated value for s^(2s)/2^s/s!
;        s= 0,   1  2    3       4       5       6
;        F=[0, 0.5, 2.0, 15.1875,170.666,2543.13,47239.2]
;     Thus cnst=(0.052/2.)*F will do like Schmelz et al.(1994)
;
; SIDE EFFECTS:
; RESTRICTIONS:
;     To have a finite L_B always, set dB/dz > dLmin(=1 gauss/E11 cm)
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)
;     09-Mar-2003  DG
;       Changed to put fastest varying indexes first to avoid maniacal
;       page faulting.
;     10-Mar-2003  DG
;       Fixed major bug in calculation of L1D, where gradient was always
;       set to minimum (dLmin = 1.e-11) because it was being compared
;       with the negative gradient.  Should have been compared to
;       abs(gradient).
;-

function mw_gr_opacity, mdl,fGHz,fk

cnst=(.052/2.)*[0,0.5,2.0,15.1875,170.666,2543.13,47239.2]

dLmin=1.0E-11
Te = mdl.Te  & Dn = mdl.Dn & z = mdl.z*1.0E5
Bz = mdl.Bz  & Bt = mdl.Bt
Br = sqrt(Bt^2-Bz^2)
An = atan(Br, Bz) ; [0,pi] but OK since we use only |sin t| |cos t|
fHz= fGHz*1.0E9

nx=n_elements(mdl.x) & ny=n_elements(mdl.y) & nz=n_elements(z)
nf=n_elements(fGHz)  & ns= n_elements(cnst)

Te3f = (tau_x = (tau_o = fltarr(nf,ns,nx,ny) ) )

for j=0,ny-1 do begin
for i=0,nx-1 do begin

  B1d =reform(Bt[i,j,*])
  L1d =B1d/(abs(deriv(z,B1d))>dLmin)
  T1d= reform(Te[i,j,*])
  D1d= reform(Dn[i,j,*])
  a1d= reform(An[i,j,*])

for q=0,nf-1 do begin

   for s=1, ns-1 do begin
      k1=fix(fk[q,s,i,j])
 	  if (k1 gt 1) and (k1 lt (nz-2)) then begin
	     fr=fk[q,s,i,j]-k1
   	     Ts=T1d[k1]*(1.-fr) + T1d[k1+1]*fr
   	     Ds=D1d[k1]*(1.-fr) + D1d[k1+1]*fr
   	     As=A1d[k1]*(1.-fr) + A1d[k1+1]*fr
   	     Ls=L1d[k1]*(1.-fr) + L1d[k1+1]*fr
	     sn=abs(sin(as) )
	     cs=abs(cos(as) )
         log_cmn = alog(cnst[s])+alog(Ds)+alog(Ls) $
                    + float(s-1)*alog(1.77E-10*Ts*sn)-alog(fHz[q])
         gr_cmn = exp(log_cmn>(-66.)<66.)
         tau_x[q,s,i,j]=gr_cmn * ( 1. + cs )^2
         tau_o[q,s,i,j]=gr_cmn * ( 1. - cs )^2
          Te3f[q,s,i,j]=Ts
	  endif
   endfor ; next harmonic
endfor ; next frequency

endfor ; next y
endfor ; next x

gr={Te:Te3f, tau_x:tau_x, tau_o:tau_o}

return, gr

end
