;+
; NAME:
;     mw_rl
; PURPOSE:
;     main program for calculating brightness temperature maps
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mw_rl,infiles[,fGHz=fGHz][,theta=theta][,phi=phi][,outfile=outfile]
; INPUTS:
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;     infile    output of mw_modlin.pro containing the input model
;     fGHz      frequency array in [GHz]	[3.0, 12.0] GHz
;     theta	theta in  degree		0 deg
;     phi	phi in degree			0 deg
;     pxl	pixel size in ["]
;     outfile	outfile name
; ROUTINES CALLED:
;     mw_transf, mw_ff_opacity, mw_gr_layers, mw_gr_opacity
;     mw_gr_plus_ff, run mw_soltb
; OUTPUTS:
;     	TBmaps for three cases of (1.ff, 2.gr, 3.both) opacities
; COMMENTS:
;	1. If theta <10 deg & phi < 10 deg, rotataion will not happen.
;	2. If pixel size is set too small, arrays may get too large!
;	3. As can be seen, you may issue each step manuary.
; SIDE EFFECTS:
;    Once being rotated, uniform grids in x,y bu z remains the same.
; RESTRICTIONS:
;    Before this pgm, run mw_modlin.pro to create the infile.
;    After this pgm, run mw_plot.pro to get the maps in ps.
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)
;     06-Apr-2003  DG
;       Changed to calculate R, L instead of X, O
;-

PRO mw_rl,infile=infile, fGHz=fGHz, theta=theta, phi=phi, pxl=pxl, $
	outfile=outfile

!except=2

if NOT keyword_set(infile) then infile='mw_modlin.sav'
if NOT keyword_set(fGHz) then fGHz=[3.0, 12.0]
if NOT keyword_set(theta) then theta = 0.0
if NOT keyword_set(phi) then phi = 0.0
if NOT keyword_set(pxl) then pxl = 4.0
if NOT keyword_set(outfile) then outfile='mw_maps.sav'
tpar=[theta,phi,pxl]


print,'1. Restoring a model from : ',infile
restore,infile ; restores a structure called "model"
print,'    Frequencies chosen:',fGHz
print,'2. Coordinate Transform and Regrid along the LOS.....'
print,'	   Rot.angle[deg]=',tpar[0:1]
print,'	   Pixel [arcsec]=',tpar[2]
mw_transf,model,tpar

print,'Save the rotated model to mdl.sav'
save,filename='mdl.sav',model,tpar,theta,phi

nx = n_elements(model.x) & ny = n_elements(model.y) & nf = n_elements(fghz)
tb0 = complexarr(nx,ny,nf) & tb1 = complexarr(nx,ny,nf) & tb2 = complexarr(nx,ny,nf)
grharm = complexarr(nf,7,nx,ny)
for kf = 0, nf-1 do begin

   print, 'Now doing frequency ',fghz[kf],'  (',kf+1,' of ',nf,')'
   print,'3. Calculating free-free opacities.....'
   ff=mw_ff_opacity_rl(model,fGHz[kf])

   print,'4. Calculating gyroresonant layers...'
   fk=mw_gr_layers(model.Bt,fGHz[kf])

   print,'5. Calculating gyroresonant opacities...'
   gr=mw_gr_opacity_rl(model,fGHz[kf],fk)

;   grharm[kf,*,*,*] = complex(reform(gr.tau_x),reform(gr.tau_o))

   print,'6. Inserting g-r into f-f opacity table...'
   both=mw_gr_plus_ff_rl(ff,gr,fk)

;   print,'7. Calculating polarization ellipse...'
;   ro = mw_e_ratio(model,fGHz[kf])

   print,'8. Determining TB with f-f opacity only.....'
   TB0[*,*,kf] = mw_soltb_rl(ff,ro)

   print,'9. Determining TB with g-r opacity only...'
   TB1[*,*,kf] = mw_soltb_rl(gr,ro)

   print,'10. Determining TB with f-f & g-r together...'
   TB2[*,*,kf] = mw_soltb_rl(both,ro)
endfor

print,'11.Saving maps to:',outfile
x=model.x & y=model.y & Bz0=reform(model.Bz[*,*,0])
save,filename=outfile,TB0,TB1,TB2,x,y,fGHz,infile,Bz0,tpar,/compress

print,'   All done. Run mw_plot to get a ps file.'


end
