;+
; NAME:
;     mw_soltb_rl
; PURPOSE:
;     insert the gyro-resonant layers into the original Te3.
;
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mw_soltb_rl, all, ro
; INPUTS:
;     all   structure containing all the quantity need for calculating
;	    brightness temperature. This means (Te, dtau) at each (x,y,z,f)
;	    After integral along z (LOS), we end up with TB(x,f,f)
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
; ROUTINES CALLED:
; OUTPUTS:
;     TB	Brightness temperature on (x,y) plane and frequency
; COMMENTS:
; 1. integration start from the sun rather than from the earth
; 2. intregartion is replaced with iterative relationship
; in this case \tau is only incremental \tau from the previous layer
; SIDE EFFECTS:
;	Returning vale, TB(x,f,f) is an complex arr to accomo. polaiz.
; RESTRICTIONS:
;     Assume that the lowest layer is entirey optically thick while
;     the highest layer is entirely thin. (implied by for k=1,nz-2...)
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)
;     09-Mar-2003  DG
;       Changed to put fastest varying indexes first to avoid maniacal
;       page faulting.
;     11-Mar-2003  DG
;       Converted to return R,L instead of X,O
;-

function mw_soltb_rl, all, Ro

nx =n_elements(all.Te[0,0,*,0])
ny =n_elements(all.Te[0,0,0,*])
nz =n_elements(all.Te[0,*,0,0])
nf =n_elements(all.Te[*,0,0,0])

TB_r=fltarr(nf,nx,ny)
TB_l=fltarr(nf,nx,ny)


for q=0,nf-1 do begin

  ; Start out with equal temperatures, set to base temperature
  TBr2d = reform(all.Te[q,0,*,*],nx,ny)
  TBl2d = reform(all.Te[q,0,*,*],nx,ny)

  ; Integrate along Z (line of sight)
  for k=1,nz-2 do begin
    Te2d = reform(all.Te[q,k,*,*],nx,ny)
 	epr2d = 1.0/exp(reform(all.tau_r[q,k,*,*],nx,ny)<66.)
 	epl2d = 1.0/exp(reform(all.tau_l[q,k,*,*],nx,ny)<66.)
 	Tr = Te2d*(1.0 - epr2d )  ; X-mode increment to Tb
 	Tl = Te2d*(1.0 - epl2d )  ; O-mode increment to Tb
; 	Dpol = reform(Ro[q,*,*,k]/(1+Ro[q,*,*,k]^2))  ; Conversion from X,O to R,L
; 	Tr = 0.5*(To+Tx) - Dpol*(To-Tx)  ; From eq 2.4-13 of Papagiannis & Kogut
; 	Tl = 0.5*(To+Tx) + Dpol*(To-Tx)  ; From eq 2.4-12 of Papagiannis & Kogut
 	; In addition to increment, opacity in this cell also attenuates
 	; brightness from layer below.  It is not clear how to convert X,O
 	; opacity to R,L, but guess that it has the same form
; 	etaur = 0.5*(epo2d+epx2d) - Dpol*(epo2d-epx2d)
; 	etaul = 0.5*(epo2d+epx2d) + Dpol*(epo2d-epx2d)
 	TBr2d = TBr2d*epr2d + Tr  ; New brightness is increment Tr plus atten.
 	TBl2d = TBl2d*epl2d + Tl
  endfor
  TB_r[q,*,*] = TBr2d
  TB_l[q,*,*] = TBl2d

endfor ; next frequency

return,complex(TB_r,TB_l)

end
