;+
; NAME:
;     mw_transf
; PURPOSE:
;     rorate to a desired viewing angle & regrid the model acc. to LOS
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mw_transf, mdl, tpar
; INPUTS:
;     mdl   Structure
;     tpar  [theta(deg), phi(deg), pixelsiz(")]
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
; ROUTINES CALLED:
; OUTPUTS:
;     mdl   Renewed structure of the input model
; COMMENTS:
;  1. No rotation if angles too small, say, less than 10 deg.
;  2. Choosing a smaller pixel size  will result in large arrays.
;  3. Most computing time in rotating & rebinning.
; SIDE EFFECTS:
;     The original model (str) is replaced with the newer quantity.
; RESTRICTIONS:
;     Once rotated, all quantities are in uniform grids rather than
;     original adaptive grids. At the moment, unsure which is better.
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)
;-

PRO mw_transf, mdl, tpar

arc_km = 7.25E2
Te=mdl.te & Dn=mdl.Dn
Bx=mdl.Bx & By=mdl.By  & Bz=mdl.Bz
 x=mdl.x  &  y=mdl. y  &  z=mdl.z
if (tpar[2] lt 1.0) then begin
   ; If pixel size is less than 1", consider it as an enlargement
   ; factor for the physical size of the region (but not height)
   x=x/tpar[2] & y=y/tpar[2]
   tpar[2] = 1.0
endif

Bt=sqrt(Bx^2+By^2+Bz^2)
nx=n_elements(x)
ny=n_elements(y)
nz=n_elements(z)
p =tpar[0] * !dtor
q =tpar[1] * !dtor
dx=tpar[2]
dy=dx

;if (tpar[0] lt 10.) and (tpar[1] lt 10.) then goto,skip

print, "   -Coord.transform: (x,y,z)--> (x',y',z)..."

; define rotation matrix
axx =  COS(p)
axy =  SIN(p)
axz =  0.
ayx = -COS(q)*SIN(p)
ayy =  COS(q)*COS(p)
ayz =  SIN(q)
azx =  SIN(q)*SIN(p)
azy = -SIN(q)*COS(p)
azz =  COS(q)

; define new planes at the rotated positions
x3d=(y3d=(z3d=fltarr(nx,ny,nz)))
for k=0,nz-1 do begin
 for j=0,ny-1 do x3d[*,j,k]=x[*]
 for i=0,nx-1 do y3d[i,*,k]=y[*]
 z3d[*,*,k]=z[k]
endfor

; rotate the coordiantes
for k=0,nz-1 do begin
; print,k,nz-1
 xl=reform(x3d[*,*,k],nx,ny) ; have to redo this at every height
 yl=reform(y3d[*,*,k],nx,ny)
 zl=reform(z3d[*,*,k],nx,ny)
 x3d[*,*,k] = axx*xl + axy*yl + axz*zl ; NB x3d is being replaced now
 y3d[*,*,k] = ayx*xl + ayy*yl + ayz*zl
endfor

; set the maximum range of the new coordinates system
; which is used to set (ps, gs) parameters for Trigrid
 xmin=min(x3d) & xmax=max(x3d)
 ymin=min(y3d) & ymax=max(y3d)
 ps=[xmin, ymin, xmax, ymax]
 gs=[dx, dy]
; mx=fix(ps[2]-ps[0])/dx) ; Right in principle, but in practice,
; my=fix(ps[3]-ps[1])/dy) ; I do not trust this way
; so actually do something ...
 xp=reform(x3d[*,*,0])
 yp=reform(y3d[*,*,0])
 fp=reform( Te[*,*,0])
 triangulate,xp,yp,tr
 hama=trigrid(xp,yp,fp,tr,gs,ps)
 mx=n_elements(hama[*,0])
 my=n_elements(hama[0,*])
; Now we know that
 xm=ps[0] + gs[0]*findgen(mx)
 ym=ps[1] + gs[1]*findgen(my)

print,'   -Coord.transfm of {T, N, B} and also re-grid...'
; Finally we may need to transform the quantities themselves, but
; Temperature, density, magnetic field strength are scalars and
; coordinate-indendependent. Only Bz needs to be transformed.

Bz = azx*Bx + azy*By + azz*Bz ; Note that Bz is replaced.

 Te_=fltarr(mx,my,nz) ; define new matrices
 Dn_=fltarr(mx,my,nz)
 Bt_=fltarr(mx,my,nz)
 Bz_=fltarr(mx,my,nz)

for k=0,nz-1 do begin

 xp=reform(x3d[*,*,k]) ; have to redo this at every height
 yp=reform(y3d[*,*,k])

 fp=reform(Te[*,*,k]) & mis=min(fp)
 Te_[*,*,k]=trigrid(xp,yp,fp,tr,gs,ps,MISSING=mis)

 fp=reform(Dn[*,*,k]) & mis=0.0
 Dn_[*,*,k]=trigrid(xp,yp,fp,tr,gs,ps,MISSING=mis)

 fp=reform(Bt[*,*,k]) & mis=min(fp)
 Bt_[*,*,k]=trigrid(xp,yp,fp,tr,gs,ps,MISSING=mis)

 fp=reform(Bz[*,*,k]) & mis=0.0
 Bz_[*,*,k]=trigrid(xp,yp,fp,tr,gs,ps,MISSING=mis)

endfor

; z  will change due to projection effect (oblique los)
z = z /cos(p)/cos(q)
; put stuff back to original var names
Te=Te_ & Dn=Dn_
Bt=Bt_ & Bz=Bz_
x=xm & y=ym

;goto,skip1
;
;skip:
;print,'   -The rotation angles are so small and un-applied. Sorry.'
;
;skip1:

z_km= z * arc_km

mdl={Te:Te, Dn:Dn, Bt:Bt, Bz:Bz, x:x, y:y, z:z_km }

print,'   -Note that now (x,y) in [arcsec] and z in [km].'
return

end
