;+
; NAME:
;     spfit
; PURPOSE:
;     Fit thermal free-free and gyroresonance spectrum
;     and derive electron temperature, emission measure
;     and magnetic field strength. This function returns
;     a vector of values for the dependent variables as
;     fitted by the function fit.
;
; CATEGORY:
;
; CALLING SEQUENCE:
;     result = spfit(f, t, p, harm = hm, info = info)
; INPUTS:
;     f     Frequency in GHz.
;     t     Brightness temperature in MK.
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;     harm  Harmonic number of gyroresonance emission. Default value is 2
; ROUTINES CALLED:
;
; OUTPUTS:
;     result    Vector of values for the dependent variables as
;               fitted by the function fit.
;     p     Vector of physical paramenters.
;           p[0] is the electron temperature in MK,
;           p[1] is the emission measure in 10^27 cm^-5 and
;           p[2] is the magnetic field strength in kG.
;           p[3] is the type parameter. 1 is gyro-resonance and 0 is free-free.
;               Actual value is log(f1 / f0) / log(f2 / f0) where tau_f(f0) = 2,
;               tau_f(f1) = tau_g(f1), tau_f(f2) = (1.e4K) / p[0].
;     info  Termination status. It is bitwise.
;           0   Successfull end.
;           1   Convergence failure.
;           2   f(tau_f = 1) at lower boundary.
;               If p[3] is small, p[0], p[1], p[2] are not reliable.
;           4   f(tau_f = 1) at upper boundary.
;               p[1], p[2] are not reliable.
;
; COMMENTS:
; SIDE EFFECTS:
; RESTRICTIONS:
; MODIFICATION HISTORY:
;     Written 20-Jul-2001 by Su-Chan Bong
;-
function spfit_func, par, spdat, tfit, dfunc = grd
    t = spdat.t
    f = spdat.f
    wgt = spdat.wgt
    nf = n_elements(f)

    tc = 1.e-2

    a = par[0]
    b = par[1]
    type = par[2]
    d = par[3]

    atc2 = 2. * a / tc
    c = b * (atc2^(-type) * 2.)^(-(d - 2.) / (2. * d))

    fc = f / c
    lnf_ = -alog(fc)
    fcd = fltarr(nf)
    good = where(d * lnf_ ge -17., ngood)
    if ngood gt 0l then fcd[good] = fc[good]^(-d)
    fb2 = (f / b)^(-2)
    tau_ = -(fb2 + fcd)
    etau = fltarr(nf)
    good = where(tau_ ge -17., ngood)
    if ngood gt 0l then etau[good] = exp(tau_[good])
    etau1 = 1. - etau

    tfit = tc * etau + a * etau1

    dt = tfit - t
    chi2 = total(wgt * dt^2)

    dcdt = wgt * dt
    dtdtau = (a - tc) * etau
    dcdtau = dcdt * dtdtau

    dcda = 2. * total(dcdt * (etau1 + dtdtau * fcd * (type * (d - 2.) / (2. * a))))
    dcdb = 2. * total(dcdtau * (fb2 * (2. / b) + fcd * (d / b)))
    dcdtype = 2. * total(dcdtau * fcd * ((d - 2.) / 2. * alog(atc2)))
    dcdd = 2. * total(dcdtau * fcd * (alog(b / f) + type / 2. * alog(atc2) - alog(2.) / 2.))

    grd = [dcda, dcdb, dcdtype, dcdd]

    return, chi2
end

function spfit_min, par, parbnd, spdat, info = info
    epstop = 1.e-4
    parmin = parbnd[*, 0]
    parmax = parbnd[*, 1]
    parran = parmax - parmin
    parn = par / parran
    parnmin = parmin / parran
    parnmax = parmax / parran

    itmax = 2000
    itdmax = 10
    it = 0
    itin = 0
    go = 1b
    chi2 = spfit_func(par, spdat, tfit, dfunc = grd)
    grdn = grd * parran
    while go and it lt itmax do begin
        it = it + 1
        parn0 = parn
        chi20 = chi2
        dir0 = -grdn / sqrt(total(grdn^2))
        itdir = 0
        godir = 1b
        bnd = where((parn0 eq parnmax and dir0 gt 0.) $
         or (parn0 eq parnmin and dir0 lt 0.), nbnd)
        if nbnd gt 0l then begin
            dir0[bnd] = 0.
            if nbnd lt 4l then dir0 = dir0 / sqrt(total(dir0^2)) $
             else begin
                godir = 0b
                go = 0b
                goto, getout
             endelse
        endif
        while godir and itdir lt itdmax do begin
            parn = parn0 + epstop * dir0 < parnmax > parnmin
            dparn = parn - parn0
            ep = sqrt(total(dparn^2))
            dir0 = dparn / ep
            if ep gt epstop * 0.9 then godir = 0b
            itdir = itdir + 1
        endwhile
        dcdp0 = total(grdn * dir0)
        chi2 = spfit_func(parn * parran, spdat, dfunc = grd)
        grdn = grd * parran
        dcdp = total(grdn * dir0)
        if dcdp - dcdp0 lt -1.e-6 * dcdp0 $
         then parn = parn0 + ep * 1.e6 * dir0 < parnmax > parnmin $
         else parn = parn0 - ep / (dcdp - dcdp0) * dcdp0 * dir0 < parnmax > parnmin
        dparn = parn - parn0 < 0.1 > (-0.1)
        parn = parn0 + dparn
        par = parn * parran
        chi2 = spfit_func(par, spdat, tfit, dfunc = grd)
        grdn = grd * parran
        if abs(chi2 - chi20) le epstop * (chi2 + chi20) / 2. then begin
            itin = itin + 1
            if itin ge 3 then go = 0
        endif else itin = 0
    endwhile
   getout:
    info = go + 2l * (parn[1] - parnmin[1] lt 0.1) $
     + 4l * (parnmax[1] - parn[1] gt 0.9)

    return, tfit
end

function spfit, f_, t_, p, harm = hm_, info = info
    if n_elements(hm_) gt 0l then hm = hm_ else hm = 2.

    f = f_
    tc = 1.e-2
    t = reform(t_) > tc
    wgt = (1. / tc)^2
    spdat = {f : f, t : t, wgt : wgt}
    tlow = 2. * tc
    tmax = max(t) > 1.1 * tlow
    a0 = tmax
    crn = where(t ge 1., ncrn)
    if ncrn gt 0l then a0 = total(t[crn]) / ncrn
    thn = where(t lt (tc * exp(-0.2) + a0 * (1. - exp(-0.2))), nthn)
    fmax = max(f, min = fmin)
    if nthn gt 0l then f1 = min(f[thn]) else f1 = fmax
    par = fltarr(4)
    par[0] = a0
    par[1] = f1 * sqrt(0.2) < fmax > fmin
    par[2] = 0.5
    par[3] = 5.

    parbnd = [[tlow, fmin, 0., 5.], $
     [tmax, fmax, 1., 20.]]
    tfit = spfit_min(par, parbnd, spdat, info = info)

    a = par[0]
    b = par[1]
    type = par[2]
    d = par[3]

    atc2 = 2. * a / tc
    c = b * (atc2^(-type) * 2.)^(-(d - 2.) / (2. * d))

    p = fltarr(4)
    p[0] = a
    p[1] = 4.8 * a^1.5 * b^2
    p[2] = c / (2.8 * hm)
    p[3] = type

    return, tfit
end